
-- Create Trip's main table
rawset(_G, "trip", {})

	-- Variables and tables --

-- Regular Trip
trip.DJUFORCE = 9 * FRACUNIT				-- Base force of the double jump. 10*FRACUNIT is a regular sonic jump.
trip.SOLJFORCE = 11*FRACUNIT				-- Sol form force of the double jump
trip.EFFECINT = 3							-- How many tics to wait between wall effect creation
trip.MAXCLIMB = 18 * FRACUNIT				-- Maximum wall speed before trip starts braking
trip.WALBRAKE = 10							-- X in ( FRACUNIT / X ) for how fast we return to wall climb speed 0
trip.STRBRAKE = 25							-- X in ( FRACUNIT / X ) for how fast we return to wall strafe speed 0
trip.WALLSIDE = 7							-- How fast trip can go left and right in a wall in FRACUNITs
trip.WALLROLL = ANG10						-- The maximum sprite roll for trip while climbing a wall
trip.BOUNCEMINTICS = TICRATE/2				-- Minimum charge time for our bounce to work
trip.BOUNCEMAXTICS = TICRATE*13/9			-- Time to get to our bounce's maximum charge
trip.BOUNCEMINSPD = FRACUNIT*15				-- Minimum speed gained from our bounce
trip.BOUNCEMAXSPD = FRACUNIT*40				-- Maximum speed gained from our bounce
trip.BOUNCEMINHEIG = FRACUNIT*8				-- Minimum height gained from a bounce
trip.BOUNCESTOPFORCE = FRACUNIT*3			-- Extra gravity force applied every tic to a bounce
trip.BOUNCELOCKPANIC = TICRATE*3			-- Stop forcing our way down if we haven't hit the floor after this time
trip.BOUNCEMAXMOMZ = 85						-- Maximum Z momentum gained from a bounce
trip.CEILSTICKRANGE = FRACUNIT*20
trip.CEILSTICKMULT = FRACUNIT/2
trip.SOLQUAKERANGE = 300*FRACUNIT
trip.SOLQUAKEHEIGHT = 200*FRACUNIT

-- Spike mode stuff
trip.SpikesGravity = FRACUNIT*6/5
trip.SpikesNormalspeedMultiplier = FRACUNIT*3/5			-- probably not needed with the curve
trip.SpikesAccelMaxThrust = FRACUNIT*3/2				-- the maximum thrust value from 0 speed
trip.SpikesAccelMinThrustModifier = FRACUNIT/6			-- % of trip.SpikesAccelMaxThrust to keep at max speed
trip.SpikesAutobrakeThrust = -(FRACUNIT*7/4)
trip.SpikesMaxFrameTics = 6
trip.SpikesMinFrameTics = 1
trip.SpikesMaxFrameTicsSpeedMultiplier = FRACUNIT*8/9

-- Super Trip
trip.FLYSPD   = FRACUNIT - FRACUNIT/6		-- How fast we float up and down
trip.FLYBRAKE = 15							-- X in ( FRACUNIT / X ) for how fast we return to momz 0
trip.MAXFLYSP = 22 * FRACUNIT				-- Maximum flight speed going up and down
trip.FLAMESPD = 15 * FRACUNIT				-- Flame jet's horizontal speed / reach
trip.FLPERIOD = 10							-- Period of the flames' sine wave
trip.FLAMPLIT = 6							-- Amplitude of the flames' sine wave
trip.FLAMEGAS = TICRATE*2					-- How many tics until super trip's flames reach their full power
trip.SUPERRAD = 24*FRACUNIT					-- Super Trip's radius
trip.SUPERHEI = 48*FRACUNIT 				-- Super Trip's height

-- Extras
trip.GREETING = false						-- Thank you so much-a for downloading our mod
trip.CUSTIDLETIME = TICRATE*6				-- Custom waiting time for trip's idle animation
trip.CUSTTEAMCOLOR = true					-- Should Trip use her custom team colors in team game modes?
trip.ALLOWSMASHBRAWL = false

-- Battlemod
trip.FIRRANGE = RING_DIST					-- Lock-on range for trip's fireballs. not always multiplied by FRACUNIT.
trip.FIRECOST = 5							-- Cost for each fireball in battle mode
trip.FIRECHARGETIME = 40					-- How long a fireball needs to be charged to become a flame breath
				   -- ^ MUST BE A MULTIPLE OF 10 BECAUSE MATH
trip.BATTLEFLAMESPD = 15 * FRACUNIT			-- Battlemod variant of flame jet speed
trip.FIRECOOLDOWN = TICRATE 				-- Cooldown for fireballs
trip.BREATHCOOLDOWN = TICRATE*5/2			-- Cooldown for fire breath
trip.AIRBUMPCOOLDOWN = TICRATE/2			-- Cooldown for being bumped in the air while charging your fire breath

-- Table for defining invalid sprite2s for flames
trip.NOFLAMETABLE = {
	"TRNS", ""
}

-- Table for flame colors
trip.flameSkins =  { SKINCOLOR_YELLOW, SKINCOLOR_ORANGE, SKINCOLOR_RED, SKINCOLOR_CRIMSON,
					 SKINCOLOR_RUBY, SKINCOLOR_VOLCANIC, SKINCOLOR_KETCHUP, SKINCOLOR_GOLD,
					 SKINCOLOR_RUST }

-- Frame order to make our wall climbing play in reverse					 
trip.inverseRoll = {D,C,B,A}
trip.fakeRoll = {A,B,C,D}

-- Table of states wherein trip is using her bigger rolling sprites
trip.donutStates = {}
trip.donutStates[S_PLAY_JUMP] = true	trip.donutStates[S_PLAY_CLIMB] = true
trip.donutStates[S_PLAY_CLING] = true	trip.donutStates[S_PLAY_TNROLL1] = true
trip.donutStates[S_PLAY_TNROLL2] = true trip.donutStates[S_PLAY_TRSPIKE] = true
trip.donutStates[S_PLAY_TSPIKEROLL] = true

-- String to bool table
local StrBool = {}
StrBool["true"] = true	StrBool["false"] = false
local DefPref = {true, true, false, false, true}

	-- Functions --

-- Checks for player validity
trip.IsValid = function(mo, ignoreSkin)
	if (not mo or not mo.valid) or
	   (not mo.player or not mo.player.valid) or
	   (maptol & TOL_NIGHTS)
		return false end

	if ignoreSkin ~= true
		if (not mo.player.TRP) or
		   (mo.skin ~= "trip")
			return false end
	end

	return true
end

-- Clamp. Why is this still not in the base SRB2 library?
trip.Clamp = function(value, minV, maxV)
	if minV > maxV then minV, maxV = maxV, minV end
    return max(minV, min(maxV, value))
end

-- Optimized ghost mobj for less FPS drops in battle mode
trip.OptimizedGhost = function(mo,colorize)
	local ghost = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_GHOST)
	ghost.fuse = TICRATE/4
	ghost.color = mo.color
	ghost.state = mo.state
	ghost.sprite = mo.sprite
	ghost.angle = mo.angle
	ghost.frame =  mo.frame
	ghost.frame = $|TR_TRANS50 
	if colorize then
		ghost.colorized = colorize
	end
	if (mo.flags & MF2_SPLAT) then
		ghost.flags2 = $|MF2_SPLAT
		ghost.renderflags = RF_SLOPESPLAT|RF_NOSPLATBILLBOARD
		P_CreateFloorSpriteSlope(ghost)
		ghost.floorspriteslope = mo.floorspriteslope
	end
	
	return ghost
end

-- We are no longer super. Let's fix some stuff.
trip.ReturnFromSuper = function(p)
	if not p.valid return end
	if p.TRP
		p.TRP.superReturn = false
		p.TRP.flaming = false
		p.TRP.superFlight = nil
		p.TRP.flameGas = 0
		if p.TRP.truesize
			if p.mo and p.mo.valid then p.mo.scale = p.TRP.truesize end
			p.TRP.truesize = nil
		end
	end
	p.charflags = $ & ~SF_NOJUMPSPIN
	p.charability2 = CA2_SPINDASH
	p.pflags = $ & ~PF_JUMPSTASIS
	p.thrustfactor = 5
	if p.mo and p.mo.valid
		p.mo.flags = $ & ~MF_NOGRAVITY
		p.mo.radius = skins["trip"].radius
		p.mo.height = skins["trip"].height
		if P_IsObjectOnGround(p.mo) and (p.cmd.buttons & BT_SPIN) then p.mo.state = S_PLAY_STND end
		if not P_IsObjectOnGround(p.mo) then p.pflags = $|PF_THOKKED end
	end
	p.powers[pw_nocontrol] = 1
end

-- Can we return from our super form here or is it too cramped?
trip.SuperAreaCheck = function(mo)
	if not mo.valid return false end
	local blocked = false
	local range = trip.SUPERRAD + 40*FRACUNIT
	local height = trip.SUPERHEI + 20*FRACUNIT
	searchBlockmap("objects", function(pmo, foundmo)
		if not foundmo.valid return end
		if (foundmo.flags & MF_SOLID) and abs(pmo.z - foundmo.z) < height
		and not (foundmo.player)
			blocked = true
			return true
		end
	end, mo, mo.x - range, mo.x + range, mo.y - range, mo.y + range)
	if not blocked
		local poscheck = P_SpawnMobjFromMobj(mo, 0,0,0, MT_TORPEDO)
		if P_MobjFlip(mo) == -1 then poscheck.flags2 = $|MF2_OBJECTFLIP end
		poscheck.target = mo
		poscheck.radius = range
		poscheck.height = height
		P_SetOrigin(poscheck, mo.x, mo.y, mo.z)
		poscheck.fuse = 1
		poscheck.cusval = 70
		poscheck.flags = $|MF_NOCLIPTHING
		if not poscheck.valid or not P_TryMove(poscheck,poscheck.x+1, poscheck.y+1, true)
			return false
		end
		return true
	end
	return false
end

// Can we go super?
trip.SuperReady = function(p)
	if (not p.powers[pw_super]
	and not p.powers[pw_invulnerability]
	and not p.powers[pw_tailsfly]
	and ((p.pflags & PF_JUMPED) or SUBVERSION >= 14)
	and not (p.powers[pw_shield] & SH_NOSTACK)
	and not (maptol & TOL_NIGHTS)
	and All7Emeralds(emeralds)
	and (p.rings >= 50))
	and trip.SuperAreaCheck(p.mo)
	and not (p.lastbuttons & BT_SPIN)
	and not (p.mo.state == S_PLAY_TRSPIKE)
	and not (p.TRP.spikemode)
		return true
	end
	return false
end

// Let's try to transform
trip.TrySuperTransform = function(p, butt)
	if not p.mo or not p.mo.valid return end
	if not trip.IsValid(p.mo) return end
	if (p.lastbuttons & butt) return end

	if trip.SuperReady(p)
		p.charflags = $|SF_SUPER
		p.pflags = $ & ~PF_SPINNING
		P_DoSuperTransformation(p)
	end
end

// Stay curled
trip.MaintainLightOrb = function(p)
	p.powers[pw_sneakers] = 0
	p.mo.state = S_PLAY_LIGHTBALL
	p.panim = PA_ROLL
	p.mo.frame = $ | FF_ADD | FF_TRANS10 | FF_FULLBRIGHT
	if not CBW_Battle
		local ghost = P_SpawnGhostMobj(p.mo)
		P_InstaThrust(ghost, FixedAngle(P_RandomRange(1, 360)*FRACUNIT), P_RandomRange(-2, 2)*FRACUNIT)
		ghost.momz = P_RandomRange(-2, 2)*FRACUNIT
		ghost.destscale = 1
	end
end

// Because 2.2.14 took a while
trip.CheckSkyHit = function(mo, line)
	if (line and (line.special == 41 or
		(line.backsector
		and line.backsector.ceilingpic == "F_SKY1"
		and line.frontsector
		and line.frontsector.ceilingpic == "F_SKY1"
		and (mo.z >= line.frontsector.ceilingheight
		or mo.z >= line.backsector.ceilingheight))))
			return true
	end
	return false
end

-- angtoint
trip.AngToInt = function(a)
	if not a return 0 end
	a = $/ANG1
	if a < 0
		a = $+360
	end
	return a
end

-- Control + direction angle
trip.GetControlAngle = function(p)
	local mo = p.mo
	local cmd = p.cmd
	
	local controlangle, directionangle
	if not ((twodlevel) or (mo.flags2 & MF2_TWOD))
		controlangle = R_PointToAngle2(0, 0, cmd.forwardmove * FRACUNIT, -(cmd.sidemove * FRACUNIT))
		directionangle = ((cmd.angleturn << 16) + controlangle)
	elseif (mo and mo.valid)	-- needed for 2d mode
		-- adjust directionangle calculation for 2d
		controlangle = 0
		if not (cmd.sidemove > 0 and mo.angle == 0) 
		and not (cmd.sidemove < 0 and mo.angle == ANGLE_180)
			-- backpedaling detected
			controlangle = ANGLE_180
		end
				
		directionangle = mo.angle + controlangle	-- use mo.angle instead for 2d
	end
	
	return directionangle or 0
end

-- Create a powerful shockwave that brings up stalagmites from the ground when transformed
trip.DoSolShockwave = function(mo)

	local range = trip.SOLQUAKERANGE
	searchBlockmap("objects", function(pmo, foundmo)
		if not foundmo.valid return end
		if foundmo.state == foundmo.info.painstate or (foundmo.flags2 & MF2_FRET) return end

		if foundmo and foundmo.health and ((foundmo.flags & MF_ENEMY) or (foundmo.flags & MF_BOSS))
		and P_IsObjectOnGround(foundmo) and not foundmo.skewered
		and abs(pmo.z - foundmo.z) < trip.SOLQUAKEHEIGHT
		and ((foundmo.type == MT_EGGGUARD) or (foundmo.flags & MF_SHOOTABLE))
			if foundmo.health-2 <= 0
				foundmo.skewered = true
				local ang = R_PointToAngle(pmo.x, pmo.y)
				local mite = P_SpawnMobjFromMobj(foundmo, cos(ang),sin(ang),0, MT_DSZSTALAGMITE)
				if P_MobjFlip(foundmo) == -1 then mite.flags2 = $|MF2_OBJECTFLIP end
				mite.spriteyscale = 1
				mite.cusval = 133780082
				mite.flags = $ & ~MF_SOLID
				mite.fuse = 30
				mite.spritexscale = FRACUNIT * 3/4
				local dirt = P_SpawnMobjFromMobj(foundmo, 2*cos(ang),2*sin(ang),0, MT_MINUSDIRT)
				if P_MobjFlip(foundmo) == -1 then dirt.flags2 = $|MF2_OBJECTFLIP end
				dirt.fuse = 30
				S_StartSound(foundmo, sfx_tridsp)
				if foundmo.health-2 <= 0
					local hitstop = P_SpawnMobjFromMobj(foundmo, -cos(ang),-sin(ang),0, MT_UNKNOWN)
					hitstop.fuse = 30
					hitstop.target = foundmo
					hitstop.cusval = 908070
					hitstop.tracer = pmo
					hitstop.flags2 = $|MF2_DONTDRAW
				end
			else
				P_DamageMobj(foundmo, pmo, pmo, 2)
			end
		end
	end, mo, mo.x - range, mo.x + range, mo.y - range, mo.y + range)

end

-- Things to be done whenever trip goes idle. Done here so it can be accessed from NIGHTS stages as well
trip.IdleCheck = function(p)
	// We just went idle. Let's make our custom waiting anim take a bit longer to trigger.
	if p.mo.state == S_PLAY_STND and (p.mo.tics >= 103 and p.mo.tics <= 105) then p.mo.tics = trip.CUSTIDLETIME end
	if p.mo.state == S_PLAY_STND and p.mo.tics == 106 then p.mo.tics = 102 end

	// Enter our "proper" waiting state...
	if p.mo.state == S_PLAY_WAIT
		S_StartSoundAtVolume(p.mo, sfx_shrpsp, 120)
		p.mo.state = S_PLAY_TWAIT1
		p.panim = PA_IDLE
	end

	// ...we'll play some fitting sound effects when appropriate...
	if p.mo.state == S_PLAY_TWAIT1 
	and ((p.mo.tics == 72) or (p.mo.tics == 64))
		S_StartSoundAtVolume(p.mo, sfx_cdfm17, 100)
	end
	if p.mo.state == S_PLAY_TWAIT1 and p.mo.tics == 44 then S_StartSoundAtVolume(p.mo, sfx_kc55, 100) end
	if p.mo.state == S_PLAY_TWAIT1 and p.mo.tics == 16 then S_StartSound(p.mo, sfx_trphew) end

	// ...and also end our custom wait state properly
	if (p.mo.state == S_PLAY_TWAIT1 or p.mo.state == S_PLAY_TWAIT2)
		p.panim = PA_IDLE
		if p.speed then p.mo.state = S_PLAY_WALK end
	end

	// Throw away our helmet when we start moving again or change states
	if not (maptol & TOL_NIGHTS) and p.TRP
		if (p.TRP.lastState == S_PLAY_TWAIT2 or (p.TRP.lastState == S_PLAY_TWAIT1 and p.TRP.lastFrame >= L))
		and p.mo.state ~= S_PLAY_TWAIT1 and p.mo.state ~= S_PLAY_TWAIT2
			local helm = P_SpawnMobjFromMobj(p.mo, 0,0,p.mo.height, MT_CRAWLASTATUE)
			helm.flags = MF_NOCLIPTHING
			local glub = (p.mo.eflags & MFE_UNDERWATER) and 3*FRACUNIT or 7*FRACUNIT
			P_SetObjectMomZ(helm, glub)
			helm.fuse = TICRATE + 15
			helm.sprite = SPR_THEL
			helm.frame = A
			helm.spriteroll = P_RandomRange(-30, 30)*ANG1
			P_InstaThrust(helm, P_RandomRange(0, 359)*ANG1, 3*FRACUNIT)
			S_StartSoundAtVolume(p.mo, sfx_kc55, 100)
		end
	end
end

-- I am not a clever man
trip.DoIO = function(p)
	local path = "client/TripPrefs.cfg"
	local file = io.openlocal(path)

	if file
		local prefs = {}
		local prefstr = {}
		local i = 1
		local invalid = false
		for pref in file:read("*a"):gmatch("%S+") do
			if i > 5 continue end
			if pref == "nil" or (pref ~= "true" and pref ~= "false" and pref ~= "mute")
				invalid = true
				prefs[i] = DefPref[i]
			else
				prefstr[i] = pref
				prefs[i] = StrBool[pref]
			end
			i = $+1
		end
		file:close()

		-- out of date or missing parameters
		if #prefs < 5 or invalid
			prefstr[1] = $ or "true"
			prefstr[2] = $ or "true"
			prefstr[3] = $ or "false"
			prefstr[4] = $ or "false"
			prefstr[5] = $ or "true"
			file = io.openlocal(path, "w+")
			file:write(prefstr[1] .. " " .. prefstr[2] .. " " .. prefstr[3] .. " " .. prefstr[4] .. " " .. prefstr[5])
			file:close()
		end

		if netgame
			COM_BufInsertText(p, "t_supermusic " .. prefstr[1] .. " true")
			trip.CUSTTEAMCOLOR = $
			COM_BufInsertText(p, "t_omniclimb " .. prefstr[3] .. " true")
			COM_BufInsertText(p, "t_disablecredits " .. prefstr[4] .. " true")
			COM_BufInsertText(p, "T_fangmusic " .. prefstr[5] .. " true")
		else
			p.TSUPERMUS = prefs[1]
			trip.CUSTTEAMCOLOR = prefs[2]
			p.TOMNICLIMB = prefs[3]
			if prefstr[4] == "mute"
				p.TRIPCREDITSMUTE = true
			else
				p.NOTRIPCREDITS = prefs[4]
				p.TRIPCREDITSMUTE = false
			end
			p.TFANGMUS = prefs[5]
		end
	else
		file = io.openlocal(path, "w+")
		file:write("true true false false true")
		file:close()
	end
end
-- Was trip loaded while we were already ingame? IO!
if consoleplayer and consoleplayer.valid and consoleplayer.playerstate == PST_LIVE then trip.DoIO(consoleplayer) end

-- aaaa
trip.UpdateIO = function(p)
	local path = "client/TripPrefs.cfg"
	local file = io.openlocal(path, "w+")

	if file
		local prefs = {
			tostring(p.TSUPERMUS),
			tostring(trip.CUSTTEAMCOLOR),
			tostring(p.TOMNICLIMB),
			tostring(p.NOTRIPCREDITS),
			tostring(p.TFANGMUS)
		}
		file:write(prefs[1] .. " " .. prefs[2] .. " " .. prefs[3] .. " " .. prefs[4] .. " " .. prefs[5])
		file:close()
	end
end

	-- Commands --

-- Toggle for Super Trip's theme
COM_AddCommand("T_supermusic", function(p, arg, sync)
	if not p or not p.valid return end
	if gamestate ~= GS_LEVEL
		CONS_Printf(p, "\x04 This command must be used while in a level.")
		return
	end

	if arg
		arg = $:lower()
		if arg == "1" or arg == "true" or arg == "yes" or arg == "on"
			if not sync then CONS_Printf(p, "\x04 Custom super music enabled.") end
			p.TSUPERMUS = true
		elseif arg == "0" or arg == "false" or arg == "no" or arg == "off"
			if not sync then CONS_Printf(p, "\x04 Custom super music disabled.") end
			p.TSUPERMUS = false
		end
		if not sync and p == consoleplayer then trip.UpdateIO(p) end
	else
		CONS_Printf(p, "\x04 Determines whether or not Trip should use custom music in her super form.")
		CONS_Printf(p, "\x82 Possible arguments: ")
		CONS_Printf(p, "\x88 1 / true / yes / on  - Enables custom super music [DEFAULT]")
		CONS_Printf(p, "\x88 0 / false / no / off - Disables custom super music")
	end
end, COM_LOCAL)

-- Toggle for Fang's custom fight theme
COM_AddCommand("T_fangmusic", function(p, arg, sync)
	if not p or not p.valid return end
	if gamestate ~= GS_LEVEL
		CONS_Printf(p, "\x04 This command must be used while in a level.")
		return
	end

	if arg
		arg = $:lower()
		if arg == "1" or arg == "true" or arg == "yes" or arg == "on"
			if not sync then CONS_Printf(p, "\x04 Custom fang battle music enabled.") end
			p.TFANGMUS = true
		elseif arg == "0" or arg == "false" or arg == "no" or arg == "off"
			if not sync then CONS_Printf(p, "\x04 Custom fang battle music disabled.") end
			p.TFANGMUS = false
		end
		if not sync and p == consoleplayer then trip.UpdateIO(p) end
	else
		CONS_Printf(p, "\x04 Determines whether or not a custom boss theme should be played in Arid Canyon Act 3.")
		CONS_Printf(p, "\x82 Possible arguments: ")
		CONS_Printf(p, "\x88 1 / true / yes / on  - Enables custom fang battle music [DEFAULT]")
		CONS_Printf(p, "\x88 0 / false / no / off - Disables custom fang battle music")
	end
end, COM_LOCAL)

-- Toggle for Trip's credits
COM_AddCommand("T_disablecredits", function(p, arg, sync)
	if not p or not p.valid return end
	if gamestate ~= GS_LEVEL
		CONS_Printf(p, "\x04 This command must be used while in a level.")
		return
	end

	if arg
		arg = $:lower()
		if arg == "1" or arg == "true" or arg == "yes" or arg == "on"
			if not sync then CONS_Printf(p, "\x04 Credits disabled.") end
			p.NOTRIPCREDITS = true
		elseif arg == "0" or arg == "false" or arg == "no" or arg == "off"
			if not sync then CONS_Printf(p, "\x04 Credits enabled.") end
			p.NOTRIPCREDITS = false
		elseif arg == "mute"
			if p.TRIPCREDITSMUTE == nil
				if not sync then CONS_Printf(p, "\x04 Credits muted.") end
				p.TRIPCREDITSMUTE = true
			else
				p.TRIPCREDITSMUTE = not p.TRIPCREDITSMUTE
				local un = p.TRIPCREDITSMUTE and "muted" or "unmuted"
				if not sync then CONS_Printf(p, "\x04 Credits " .. un .. ".") end
			end
		end
		if not sync and p == consoleplayer then trip.UpdateIO(p) end
	else
		CONS_Printf(p, "\x04 Stops Trip's credits from appearing when the character is first selected")
		CONS_Printf(p, "\x82 Possible arguments: ")
		CONS_Printf(p, "\x88 1 / true / yes / on  - Disables Trip's credits")
		CONS_Printf(p, "\x88 mute  - Enable but mute Trip's credits")
		CONS_Printf(p, "\x88 0 / false / no / off - Enables Trip's credits [DEFAULT]")
	end
end, COM_LOCAL)

-- Toggle for Trip's custom team colors
COM_AddCommand("T_customteamcolor", function(p, arg)
	if not p or not p.valid return end
	if gamestate ~= GS_LEVEL
		CONS_Printf(p, "\x04 This command must be used while in a level.")
		return
	end

	if arg
		arg = $:lower()
		if (arg == "1" or arg == "on" or arg == "yes" or arg == "true")
			CONS_Printf(p, "\x04 Trip will use special team colors in team modes.")
			trip.CUSTTEAMCOLOR = true
		elseif (arg == "0" or arg == "off" or arg == "no" or arg == "false")
			CONS_Printf(p, "\x04 Trip will use the default team colors in team modes.")
			trip.CUSTTEAMCOLOR = false
		end
		if p == consoleplayer then trip.UpdateIO(p) end
	else
		CONS_Printf(p, "\x04 Determines whether or not Trip will use custom red and blue team colors in team modes.")
		CONS_Printf(p, "\x82 Possible arguments: ")
		CONS_Printf(p, "\x88 1 / true / yes / on  - Enables custom team colors [DEFAULT]")
		CONS_Printf(p, "\x88 0 / false / no / off - Disables custom team colors")
	end
end,COM_ADMIN)

-- Allow Trip to climb walls from any direction
COM_AddCommand("T_omniclimb", function(p, arg, sync)
	if not p or not p.valid return end
	if gamestate ~= GS_LEVEL
		CONS_Printf(p, "\x04 This command must be used while in a level.")
		return
	end

	if arg
		arg = $:lower()
		if (arg == "1" or arg == "on" or arg == "yes" or arg == "true")
			if not sync then CONS_Printf(p, "\x04 Trip will now climb walls regardless of your directional input.") end
			p.TOMNICLIMB = true
		elseif (arg == "0" or arg == "off" or arg == "no" or arg == "false")
			if not sync then CONS_Printf(p, "\x04 Trip must now push against walls in front of her to climb them.") end
			p.TOMNICLIMB = false
		end
		if not sync and p == consoleplayer then trip.UpdateIO(p) end
	else
		CONS_Printf(p, "\x04 Allows Trip to latch onto any nearby walls while revved up regardless of them being in front of you.")
		CONS_Printf(p, "\x82 Possible arguments: ")
		CONS_Printf(p, "\x88 1 / true / yes / on  - Enables omnidirectional wall climbing")
		CONS_Printf(p, "\x88 0 / false / no / off - Disables omnidirectional wall climbing [DEFAULT]")
	end
end)

	-- Misc --

-- Trip's ringslinger neo loadout
local rs_loaded = false
addHook("ThinkFrame", do
    if RingSlinger and not rs_loaded
        rs_loaded = true;
        
        --Trip rock n' spitty fire
        RingSlinger.Skins["trip"] = {
            ammo = 70,--Maximum ammo.
            weapons = {RSWPN_FLAME, RSWPN_STONE}--Main, offhand
        }
    end
end)

-- Terra Trip!
if not solchars
    rawset(_G, "solchars", {})
end
solchars["trip"] = {SKINCOLOR_TERRA_TRIP_1, 2}

-- Sync any admin-set variables
addHook("NetVars", function(net)
	trip.CUSTTEAMCOLOR = net(trip.CUSTTEAMCOLOR)
	trip.ALLOWSMASHBRAWL = net(trip.ALLOWSMASHBRAWL)
end)

--Kirby compatibility!
if not(kirbyabilitytable)
    rawset(_G, "kirbyabilitytable", {})
end
kirbyabilitytable["trip"] = 6
