
local FBColors = { SKINCOLOR_APRICOT, SKINCOLOR_ORANGE, SKINCOLOR_TANGERINE, SKINCOLOR_ORANGE }


-- Creates flame pillars that stick around for a little while
local function CreateLingeringFlame(mo)
	local linger = P_SpawnMobjFromMobj(mo, 0, 0, 0,  MT_FLAME)
	linger.flags = $ & ~MF_NOGRAVITY & ~MF_PAIN & ~MF_FIRE
	linger.state = S_TTORCH
	linger.colorized = true
	linger.color = (gametyperules & GTR_TEAMS) and mo.color or trip.flameSkins[ P_RandomRange(1,#trip.flameSkins) ]
	linger.scale = 1
	linger.destscale = P_RandomRange(2,6) * FRACUNIT/4
	linger.frame = $|FF_FULLBRIGHT
	if mo.target and mo.target.valid
		linger.destscale = FixedMul($, mo.target.scale)
	end
	linger.scalespeed = FRACUNIT/20
	P_Thrust(linger, mo.angle + FixedAngle( P_RandomRange(-15,15) * FRACUNIT ), P_RandomRange(2,6)*FRACUNIT )

	linger.corona = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_UNKNOWN)
	linger.corona.colorized = true
	linger.corona.color = linger.color
	linger.corona.sprite = SPR_FLAM
	linger.corona.frame = M|FF_TRANS70|FF_ADD|FF_FULLBRIGHT
	linger.corona.desync = P_RandomRange(-3,3)
	linger.corona.target = linger
end

-- Let's make super trip's flames dance
local function FlameThinker(mo)
	if not mo or not mo.valid return end

	if not mo.cusval
		local sineWave = trip.FLAMPLIT * sin( FixedAngle((leveltime%360)*FRACUNIT) * trip.FLPERIOD )
		sineWave = FixedMul($, mo.scale)
		if mo.LR then sineWave = -$ end
		if trip.IsValid(mo.target)
			sineWave = FixedMul($, (FRACUNIT - FRACUNIT/4) - mo.target.player.TRP.flameGas)
		end
		local sideX = mo.x + (sineWave/FRACUNIT)*cos(mo.angle + ANGLE_90)
		local sideY = mo.y + (sineWave/FRACUNIT)*sin(mo.angle + ANGLE_90)

		P_TryMove(mo, sideX , sideY)

		-- Create some lingering flames as well
		if (not (mo.eflags & MFE_UNDERWATER)) and mo.tics == 9 and not (leveltime%5)
			CreateLingeringFlame(mo)
		end
	end
	P_SetObjectMomZ(mo, mo.cusval == 0 and (FRACUNIT * 3/4) or (FRACUNIT/4), true)
	mo.frame = $|FF_FULLBRIGHT
	mo.extravalue1 = $+1
	if mo.extravalue1 == 15 then mo.flags = $ & ~MF_MISSILE end

	-- Extra unruly flame particles created as ghosts every other tic
	if not (leveltime%2)
		local ghost = nil
		if (mo.eflags & MFE_UNDERWATER) 
			if not (leveltime%4) 
				ghost = P_SpawnMobjFromMobj(mo, 0,0,0, MT_SMALLBUBBLE)
				S_StartSound(ghost, P_RandomRange(sfx_bubbl1,sfx_bubbl5))
				ghost.fuse = 20
			end
		else
			ghost = CBW_Battle and trip.OptimizedGhost(mo) or P_SpawnGhostMobj(mo)
		end
		if ghost
			P_SetObjectMomZ(ghost, P_RandomRange(-3,3)*FRACUNIT)
			P_Thrust(ghost, ghost.angle + FixedAngle(P_RandomRange(90,270)*FRACUNIT), 5*FRACUNIT )
			ghost.scale = mo.scale
		end
		mo.frame = $+FF_TRANS10
	end
end

-- Trip's flames hit a wall. Move them a bit and check if the wall they hit is bustable every 10 tics.
local function FlameBlocked(mo, blocker, line)
	if not mo or not mo.valid return end
	if line
		P_BounceMove(mo)
		P_InstaThrust(mo, mo.angle, 5*FRACUNIT )
		if not (leveltime%10)
			for _, sec in ipairs({line.frontsector, line.backsector})
				for rover in sec.ffloors()
					if not (rover.flags & FF_EXISTS) or not (rover.flags & FF_BUSTUP) then continue end
					if mo.z >= rover.topheight
					or mo.z + mo.height < rover.bottomheight
						continue
					end
					EV_CrumbleChain(sector, rover)
				end
			end
		end
		return true
	end
end

-- Allow Trip's super flames to break spikes
local function FlameSpikeBust(flame, spike)
	if not flame.valid return end
	if not spike or not spike.valid return end

	
	if (spike.type ==  MT_SPIKE or spike.type == MT_WALLSPIKE)
	and abs(flame.z - spike.z) <= flame.height
		P_KillMobj(spike)
	end

	-- reusing this don't @ me
	if spike.type == MT_EGGROBO1
	and abs(flame.z - spike.z) <= flame.height
	and not spike.guessIllDie
		spike.guessIllDie = 15
		spike.holdStill = {spike.x, spike.y, spike.z, spike.angle}
		spike.flags = $ & ~MF_SPECIAL
		if flame.target and flame.target.valid and flame.target.player
			local score = P_SpawnMobj(spike.x, spike.y, spike.z+spike.height, MT_SCORE)
			P_SetObjectMomZ(score, FRACUNIT/50)
			score.state = $+1
			P_AddPlayerScore(flame.target.player, 200)
		end
	end
end

-- Fuck 'em minecarts
local function SaloonBurner(flame, door)
	if not flame.valid return end
	if door and door.valid and (door.type == MT_SALOONDOOR or door.type == MT_SALOONDOORCENTER)
		if door.type == MT_SALOONDOORCENTER then door.flags = $ & ~MF_SOLID return end
		if door.burnt then return end
		door.flags = $ & ~MF_SOLID|MF_NOCLIP|MF_NOCLIPTHING|MF_NOCLIPHEIGHT
		door.colorized = true
		door.color = SKINCOLOR_BLACK
		door.fuse = 35
		door.burnt = true
		S_StartSound(door, sfx_wbreak)
	end
end

-- Preventing our coronas from sticking around should their parent object be removed
local function BurntDoorThinker(mo)
	if not mo or not mo.valid return end

	if mo.burnt
		mo.flags2 = (leveltime%2) and $|MF2_DONTDRAW or $ & ~MF2_DONTDRAW
		if not (leveltime%2)
			local xy = P_RandomRange(-50, 50)
			local z = P_RandomRange(0, 100)*FRACUNIT
			local smoke = P_SpawnMobjFromMobj(mo, xy*cos(mo.angle), xy*sin(mo.angle), z, MT_SMOKE)
			P_SetObjectMomZ(smoke, 5*FRACUNIT)
		end
	end
end

-- Thinker for lingering flames to fade them out
local function LingeringThinker(mo)
	if not mo or not mo.valid return end
	if mo.state == S_TTORCH
	
		if (mo.eflags & MFE_UNDERWATER)
			P_RemoveMobj(mo)
			return
		end

		if mo.corona
			if not (mo.frame & FF_FULLBRIGHT) then mo.frame = $|FF_FULLBRIGHT end
			mo.corona.scale = (mo.scale - mo.scale/3) + sin( FixedAngle(leveltime%360) * FRACUNIT * (8 + mo.corona.desync) )/20
			P_SetOrigin(mo.corona, mo.x, mo.y, mo.z + mo.height)
		end

		if mo.tics < 7
			mo.frame = $ + FF_TRANS10
			mo.spritexscale = $ - $/5
			mo.spriteyscale = $ + $/20
			if mo.corona
				if (mo.tics == 6 or mo.tics == 5) then mo.corona.frame = $ + FF_TRANS10 end
				if mo.tics == 4
					P_RemoveMobj(mo.corona)
					mo.corona = nil
				end
			end
		end

		if mo.tics <= 1
			P_RemoveMobj(mo) 
			return 
		end

	end
end

-- Preventing our coronas from sticking around should their parent object be removed
local function CoronaThinker(mo)
	if not mo or not mo.valid return end

	if mo.sprite == SPR_FLAM
		if not (mo.frame & FF_FULLBRIGHT) then mo.frame = $|FF_FULLBRIGHT end
		if not mo.target then P_RemoveMobj(mo) end
		return
	elseif mo.sprite == SPR_TRSR
		mo.frame = $+FF_TRANS10
	end

	if mo.cusval == 908070 and mo.target and mo.target.valid
	and mo.tracer and mo.tracer.valid
		mo.target.spritexoffset = (leveltime%2) and 3*FRACUNIT or -3*FRACUNIT
		P_MoveOrigin(mo.target, mo.x, mo.y, mo.target.z)
		if mo.fuse == 26
			local floor = (P_MobjFlip(mo.target) == 1) and mo.target.floorz + 20*FRACUNIT or mo.target.ceilingz - mo.target.height - 20*FRACUNIT
			mo.target.z = floor
			mo.target.flags = MF_ENEMY|MF_NOGRAVITY
			mo.target.spriteroll = P_RandomRange(-20,20)*ANG1
			mo.target.momx = 0
			mo.target.momy = 0
			mo.target.momz = 0
		end
		if mo.fuse <= 10
			if mo.tracer and mo.tracer.valid then P_KillMobj(mo.target, mo.tracer, mo.tracer)
			else P_DamageMobj(mo.target, nil, nil) end
			mo.target = nil
			mo.tracer = nil
			mo.cusval = 0
			P_RemoveMobj(mo)
			return
		end
	end
end

-- Make our helmet vanish after being tossed out
local function HelmThinker(mo)
	if not mo or not mo.valid return end

	if mo.sprite == SPR_THEL
		mo.flags2 = (leveltime%2) and ($|MF2_DONTDRAW) or ($ & ~MF2_DONTDRAW)
		if P_IsObjectOnGround(mo) then mo.spriteroll = 0 end
	end
end

local function StompThinker(mo)
	if not mo or not mo.valid return end

	if mo.sprite == SPR_TSCI
		mo.z = mo.cusval
		if mo.fuse < 5 then mo.frame = $+FF_TRANS10 end
	end
	if mo.cusval == 1 and mo.fuse <= 10 then mo.frame = $+FF_TRANS10 end
end

-- Shrink a stalagmite created by Terra Trip's bounce
local function StalagThinker(mo)
	if not mo or not mo.valid return end

	if mo.cusval == 133780082
		if mo.fuse > 27 then mo.spriteyscale = min($+FRACUNIT/3, FRACUNIT)
		elseif mo.fuse <= 5 then mo.spriteyscale = max($-FRACUNIT/5, 1) end
	end
end

local function SuperSparkleThinker(mo)
	if not mo or not mo.valid return end

	if mo.cusval == 6090420
		if mo.tracer and mo.tracer.valid
			mo.momz = FRACUNIT * P_MobjFlip(mo.tracer)
		end
		if mo.tics == 10 then mo.destscale = 1 end
		if mo.tics < 8 then mo.frame = $ + FF_TRANS10 end
	end
end

local nonsense = {FF_TRANS90, FF_TRANS80, FF_TRANS70, FF_TRANS60, FF_TRANS50}

-- Battle mode fireballs!
local function BattleFireballThinker(mo)
	if mo.alternator == nil then mo.alternator = true end
	if mo.tics < states[S_TBBOUNCE].tics - 4
		local trail = (mo.eflags & MFE_UNDERWATER) and P_SpawnMobjFromMobj(mo, 0, 0, mo.height/2, MT_SPINDUST) or trip.OptimizedGhost(mo)
		trail.scale = mo.scale
		if trail.type == MT_SPINDUST
			P_SetObjectMomZ(trail, P_RandomRange(-3,3)*FRACUNIT)
			P_Thrust(trail, trail.angle, P_RandomRange(-8, 8)*FRACUNIT )
			trail.frame = $ | FF_TRANS60
			trail.tics = 10
		else
			trail.scalespeed = FRACUNIT/8
			trail.destscale = 1
		end
	end

	if not (leveltime%5) then mo.alternator = not $ end
	--mo.color = FBColors[ leveltime%#FBColors + 1 ]
	mo.bounces = $ or 5
	mo.spriteroll = $ - ANG30
	if mo.tics > 10
		mo.destscale = mo.alternator and mo.origscale-mo.origscale/4 or mo.origscale+mo.origscale/4
	else
		mo.destscale = 1
	end
	if P_IsObjectOnGround(mo) and not (mo.eflags & MFE_UNDERWATER)
		P_SetObjectMomZ(mo, ((leveltime%3)+4)*FRACUNIT )
		S_StartSound(mo, sfx_s3k43)
		local addAngle = 0
		for i=1,8 do
			local flame = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_UNKNOWN)
			flame.frame = $ + FF_TRANS50 | FF_ADD | FF_FULLBRIGHT
			P_SetOrigin(flame, flame.x, flame.y, P_MobjFlip(mo) == 1 and (mo.floorz + flame.height/2) or (mo.ceilingz))
			flame.angle = mo.angle + addAngle
			addAngle = $ + FixedAngle( 22 * FRACUNIT + FRACUNIT/2 )
			P_InstaThrust(flame, flame.angle, 3*FRACUNIT )
			flame.flags = $ & ~MF_NOGRAVITY
			flame.sprite = SPR_SFLM
			flame.colorized = true
			flame.color = (gametyperules & GTR_TEAMS) and mo.color or SKINCOLOR_VOLCANIC
			flame.scale = mo.scale + mo.scale/2
			flame.destscale = 1
			flame.tics = 20
			P_SetObjectMomZ(flame, P_RandomRange(2,4)*FRACUNIT )
		end

		mo.bounces = $-1
		if not mo.bounces then mo.state = S_TBFADE end
	end
end

-- The fireball touched a wall, so let's bounce and make some smaller flames
local function BattleFireballBlocked(mo, blocker, line)
	if not mo or not mo.valid return end
	if blocker return end
	if line
		P_BounceMove(mo)
		S_StartSound(mo, sfx_s3k43)
		for i=1,8 do
			local flame = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_UNKNOWN)
			flame.frame = $ + FF_TRANS50 | FF_ADD
			flame.angle = mo.angle + ANGLE_180
			P_InstaThrust(flame, flame.angle, P_RandomRange(3,5)*FRACUNIT )
			flame.flags = $ & ~MF_NOGRAVITY
			flame.sprite = SPR_SFLM
			flame.colorized = true
			flame.color = (gametyperules & GTR_TEAMS) and mo.color or SKINCOLOR_VOLCANIC
			flame.scale = mo.scale + mo.scale/2
			flame.destscale = 1
			flame.tics = 20
			P_SetObjectMomZ(flame, P_RandomRange(1,5)*FRACUNIT )
		end
		return true
	end
end

local function EggroboBump(mo, pmo)
	if not mo or not mo.valid return end
	if not pmo or not pmo.valid or not pmo.player return end

	if not trip.IsValid(pmo) return end

	if pmo.player.powers[pw_super] and not mo.guessIllDie
		mo.guessIllDie = TICRATE
		mo.holdStill = {mo.x, mo.y, mo.z, mo.angle}
		P_Thrust(pmo, R_PointToAngle2(mo.x, mo.y, pmo.x, pmo.y), 20*FRACUNIT)
		mo.flags = $ & ~MF_SPECIAL
		local score = P_SpawnMobj(mo.x, mo.y, mo.z+mo.height, MT_SCORE)
		P_SetObjectMomZ(score, FRACUNIT/50)
		score.state = $+1
		P_AddPlayerScore(pmo.player, 200)
		return true
	end
end

local function EggroboDeath(mo)
	if not mo or not mo.valid return end

	if mo.guessIllDie
		mo.target = nil
		mo.tracer = nil
		if mo.holdStill == nil then mo.holdStill = {mo.x, mo.y, mo.z, mo.angle} end
		P_SetOrigin(mo, mo.holdStill[1], mo.holdStill[2], mo.holdStill[3])
		mo.angle = mo.holdStill[4]

		mo.flags2 = (leveltime%2) and $|MF2_DONTDRAW or $ & ~MF2_DONTDRAW
		mo.spritexoffset = P_RandomRange(-3, 3)*FRACUNIT
		mo.spriteyoffset = P_RandomRange(-3, 3)*FRACUNIT

		local rad = mo.radius/FRACUNIT
		local rad2 = P_RandomRange(-rad, rad)*FRACUNIT
		rad = P_RandomRange(-rad, rad)*FRACUNIT
		local hei = mo.height/FRACUNIT
		hei = P_RandomRange(0, hei)*FRACUNIT
		local boom = P_SpawnMobjFromMobj(mo, rad, rad2, hei, MT_BOSSEXPLODE)
		boom.scale = FRACUNIT/3
		boom.frame = $|FF_TRANS50

		if not (leveltime%3) then S_StartSound(mo, sfx_cybdth) end

		mo.guessIllDie = $-1
		if not mo.guessIllDie
			hei = (mo.height/2) - FixedMul(12*FRACUNIT, mo.scale)
			local lastboom = P_SpawnMobjFromMobj(mo, 0, 0, hei, MT_SONIC3KBOSSEXPLODE)
			lastboom.scale = mo.scale*3/2
			lastboom.frame = $|FF_TRANS50
			S_StartSound(lastboom, sfx_cybdth)
			P_RemoveMobj(mo)
			return
		end
	end
end

addHook("MobjThinker", FlameThinker, MT_TRIPFLAME)
addHook("MobjThinker", LingeringThinker, MT_FLAME)
addHook("MobjThinker", CoronaThinker, MT_UNKNOWN)
addHook("MobjThinker", StompThinker, MT_THOK)
addHook("MobjThinker", StalagThinker, MT_DSZSTALAGMITE)
addHook("MobjThinker", SuperSparkleThinker, MT_BOXSPARKLE)
addHook("MobjThinker", StompThinker, MT_FAKEROCK)
addHook("MobjThinker", HelmThinker, MT_CRAWLASTATUE)
addHook("MobjThinker", BurntDoorThinker, MT_SALOONDOOR)
addHook("MobjThinker", BattleFireballThinker, MT_TRIPBALLS)
addHook("MobjThinker", EggroboDeath, MT_EGGROBO1)
addHook("TouchSpecial", EggroboBump, MT_EGGROBO1)
addHook("MobjMoveBlocked", BattleFireballBlocked, MT_TRIPBALLS)
addHook("MobjMoveBlocked", FlameBlocked, MT_TRIPFLAME)
addHook("MobjMoveCollide", FlameSpikeBust, MT_TRIPFLAME)
addHook("MobjMoveCollide", SaloonBurner, MT_TRIPFLAME)