--yes, this code is from espio, and yes it's with permission

local function rollAnimating(p)
	local mo = p.mo
	if (mo.state != S_PLAY_TNROLL1 and mo.state != S_PLAY_TNROLL2) then return end
	
	-- recreate how S_PLAY_ROLL animates
	local speed = FixedDiv(p.speed, FixedMul(mo.scale, mo.movefactor))	-- for some reason zoomtubes animate slower if they're slow as well
	if speed > 16<<FRACBITS
		mo.tics = min($, 1)
	end
	p.panim = PA_ROLL
end

local PlayerHeight = function(p)
	local mo = p.mo
	if (not mo) or mo.skin != "trip" or p.trip_normalroll then return end
	
	-- force normal roll when bouncing with the bubble shield
	if mo.state == S_PLAY_FALL and p.secondjump == UINT8_MAX
	and p.pflags & PF_JUMPED and p.pflags & PF_THOKKED
	and not (p.pflags & (PF_NOJUMPDAMAGE|PF_SPINNING|PF_STARTJUMP))
		mo.state = S_PLAY_TNROLL1
		p.trip_normalroll = true
		p.charflags = $ | SF_NOJUMPSPIN
		return P_GetPlayerHeight(p)
	end
	
	-- always allow a normal roll under certain conditions
	if mo.state == S_PLAY_ROLL
	and (p.powers[pw_carry] or p.pflags & PF_SHIELDABILITY
	or mo.eflags & MFE_SPRUNG -- spin boosters
	or P_PlayerTouchingSectorSpecial(p, 4, 7)) -- force spin
		p.trip_normalroll = true
		p.charflags = $ | SF_NOJUMPSPIN
		return
	end
	
	-- in a roll...
	if not ((mo.state == S_PLAY_ROLL or mo.state == S_PLAY_JUMP
	or mo.state == S_PLAY_SPINDASH or mo.state == S_PLAY_TRSPIKE
	or mo.state == S_PLAY_TNROLL1 or mo.state == S_PLAY_TNROLL2
	or mo.state == S_PLAY_TSPIKEROLL)
	and P_PlayerShouldUseSpinHeight(p)) then return end

	local height = P_GetPlayerHeight(p)
	
	if mo.state == S_PLAY_TSPIKEROLL then return height end
	
	-- no crushing, use donut roll
	if (mo.ceilingz - mo.floorz >= height)
		-- Some scripts will try to force us into a rolling state in the air. Let's not fight them.
		if mo.state == S_PLAY_ROLL and P_IsObjectOnGround(mo)
			mo.state = p.pflags & PF_SPINNING and S_PLAY_TNROLL2 or S_PLAY_TNROLL1
		end
		-- Battle spring jank
		if CBW_Battle and (mo.eflags & MFE_SPRUNG) and P_IsObjectOnGround(mo)
			mo.state = S_PLAY_ROLL
			return
		end
		return height
		
	-- uh oh! emergency rolling!
	else
		if mo.state != S_PLAY_ROLL
			mo.state = S_PLAY_ROLL
		end
		p.trip_normalroll = true
		p.charflags = $ | SF_NOJUMPSPIN
	end
end

local spinTrail = function(p)
	local mo = p.mo
    
    -- determine spin state
    PlayerHeight(p)
    
    -- let us use trip's special roll again after we stop rolling normally
    if p.trip_normalroll and mo.state != S_PLAY_ROLL
        p.trip_normalroll = false
        p.charflags = $ &~ SF_NOJUMPSPIN
    end
end

local TripSpinHandler = function(p)
	local mo = p.mo
	if not ((mo and mo.valid) and mo.health and mo.skin == "trip")
		-- Make sure we reset normalroll
		if p.trip_normalroll
			p.trip_normalroll = false
			if skins[p.skin].name == "trip"
				p.charflags = $ &~ SF_NOJUMPSPIN
			end
		end
		return
	end
	
	spinTrail(p)
	rollAnimating(p)

end

addHook("ThinkFrame", function()
	for p in players.iterate do
		TripSpinHandler(p)
	end
end)
addHook("PlayerHeight", PlayerHeight)