

trip.WallReset = function(p)
	local mo = p.mo

	p.TRP.wallclimb, mo.wallclimb = nil, nil
	p.lastlinehit, p.lastsidehit = -1, -1
end


trip.AbilityCheck = function(p)
	local mo = p.mo
	
	local abilitycheck = p.charability != CA_NONE
	
	if abilitycheck and ( (not p.TRP) or (not p.TRP.wallclimb) )
		return false
	end
	
	return true
end



-- Wall cling code taken from Espio's code with permission from its author, Snu
local function Tr_CheckWall(p, line, side)
	local mo = p.mo
	
	-- really easy one, dont cling to noclimb walls
	if not (allownoclimb)
		if (line.flags & ML_NOCLIMB) then return false end
	end

	if P_InQuicksand(mo) then return end
	
	-- horizon line
	if (line.special == 41) then return end
	
	-- sector checks
	if not (line.backsector) then return true end	-- apparently in these situations source always returns true?
	local sec = (line.backsector and not P_PointOnLineSide(mo.x, mo.y, line)) and line.backsector or line.frontsector
	//print((sec == line.backsector) and "backsector" or "frontsector")
	
	local canclimb = false
	local checkz = {mo.z, mo.z + mo.height}
	local floorz = P_GetZAt(sec.f_slope, mo.x, mo.y, sec.floorheight)
	local ceilingz = P_GetZAt(sec.c_slope, mo.x, mo.y, sec.ceilingheight)
	
	-- floor check
	if checkz[2] < floorz
		canclimb = true
	end
	
	-- ceiling checks
	local thokbarrier = nil
	if P_CheckSkyHit then thokbarrier = P_CheckSkyHit(mo, line) -- use vanilla thok barrier check if it exists
	else thokbarrier = trip.CheckSkyHit(mo, line) end -- otherwise use our ported version
	if ((checkz[1] >= ceilingz) and not thokbarrier) 
		canclimb = true
	end
	
	-- FOF checks
	for rover in sec.ffloors()
		if rover.flags & FF_EXISTS
		and rover.flags & FF_BLOCKPLAYER
		and checkz[1] > P_GetZAt(rover.b_slope, mo.x, mo.y, rover.bottomheight)
		and checkz[2] < P_GetZAt(rover.t_slope, mo.x, mo.y, rover.topheight)
			canclimb = true
			break
		end
	end
	
	-- if we're here, line checks went right
	return canclimb
end
		
-- Actual wall sticking goes here
trip.DoWallClimb = function(p)
	local mo = p.mo
	if not trip.AbilityCheck(p) then return end
	if p.TRP.wallbuffer then return end
	
	if (p.gotflag) or (p.gotflagdebuff) or (p.gotcrystal) return end -- Rush: this checks for wall sticking if she has the flag
	if CBW_Battle and p.exhaustmeter <= 0 then return end	-- Rush: Check For If we lost exhaust then dont recling stoopid

	if (mo.wallclimb) then return end
	if (p.lastlinehit != -1)	-- we hit a line?
		
		local l, s = lines[p.lastlinehit], sides[p.lastsidehit]
		
		if (Tr_CheckWall(p, l, s))	-- make sure everything is good here
			
			-- face the wall
			local whichside = P_PointOnLineSide(mo.x, mo.y, l)
			local angle = R_PointToAngle2(l.v1.x, l.v1.y, l.v2.x, l.v2.y)
			angle = $ + (ANGLE_90 * (whichside and -1 or 1))
					
			mo.angle = angle
			p.drawangle = angle
				
			-- make us start running up it (this is a dirty climbing hack)
			p.charability = CA_GLIDEANDCLIMB
			p.climbing = 5
			p.TRP.wallclimb, mo.wallclimb = true, true	-- trust me there's a reason we have two saved
			S_StartSound(mo, sfx_s23d)
			p.TRP.sineBrake = 0
				
			-- shield stuff
			p.pflags = $ & ~(PF_SHIELDABILITY)
			p.secondjump = 0
			
			-- wall momentum
			if (not p.TRP.bounced)
				if p.TRP.spikemode and not ((twodlevel) or (p.mo.flags2 & MF2_TWOD))
					p.TRP.wallSpeed = FixedHypot(mo.momx, mo.momy) * P_MobjFlip(p.mo)
					if p.cmd.forwardmove < 0 then p.TRP.wallSpeed = $*-1 end
				elseif p.cmd.forwardmove >= 0
					p.TRP.wallSpeed = mo.momz * P_MobjFlip(p.mo) + ( p.TRP.wallTime and p.speed/2 or 1 )
				end
			end
			p.TRP.bounced = false
			if P_MobjFlip(mo) == -1 then p.TRP.spikemode = false end
			if mo.state == S_PLAY_TNROLL2 then mo.state = S_PLAY_CLIMB end

			return
		end
		
		-- reset these to check for wall collision again
		p.lastlinehit, p.lastsidehit = -1, -1
		
	end
end

trip.DoWallStones = function(mo, fromTop, ceiling)
	local radius = {}
		radius[1] = (mo.radius/2)/FRACUNIT * cos(mo.angle)
		radius[2] = (mo.radius/2)/FRACUNIT * sin(mo.angle)
	if ceiling
		radius[1] = mo.radius/FRACUNIT * cos(mo.angle+ANGLE_180)
		radius[2] = mo.radius/FRACUNIT * sin(mo.angle+ANGLE_180)
	end
	local rocc = P_SpawnMobjFromMobj(mo, 
									 radius[1], 
									 radius[2], 
									 (fromTop and mo.player.spinheight * P_MobjFlip(mo) or 1),
	MT_FAKEROCK)
	if rocc
		local scaleRand = P_RandomRange(1, 3)
		
		rocc.state = S_ROCKCRUMBLEJ
		rocc.flags = $ & ~MF_PAIN
		rocc.scale = (FRACUNIT - FRACUNIT/3)/scaleRand
		rocc.fuse = TICRATE
		rocc.angle = mo.angle + ANGLE_90
		rocc.destscale = 1
		rocc.scalespeed = FRACUNIT/200

		if fromTop then P_SetObjectMomZ(rocc, abs(mo.player.TRP.wallSpeed/3)) end
		P_Thrust(rocc, rocc.angle, P_RandomRange(-4, 4) * FRACUNIT/2)
		
		if mo.player == consoleplayer then P_StartQuake(3*FRACUNIT, 5) end
		S_StartSound(mo, (leveltime%2) and sfx_rocks2 or sfx_rocks1)
		return rocc
	else
		return nil
	end
end

local holes = {A, B, C, D}

trip.WallClimbMovement = function(p)

	-- Up and down movement
	local shouldBrake = ((p.TRP.wallSpeed > 0 and p.cmd.forwardmove < 0) or (p.TRP.wallSpeed < 0 and p.cmd.forwardmove > 0))
	if p.TRP.wallreverse then shouldBrake = false end
	
	p.TRP.wallSpeed = $ + (p.cmd.forwardmove > 0 and 1 or -1) * (FRACUNIT/5) * (p.TRP.wallreverse and -1 or 1)

	-- Create visual and audio effects based on our speed
	if abs(p.TRP.wallSpeed) > 3 * FRACUNIT
		if not (leveltime%3) 
			local boo = P_SpawnGhostMobj(p.mo)
			boo.scale = p.mo.scale
		end
		if not (leveltime%3)
			trip.DoWallStones(p.mo, p.TRP.wallSpeed < -10*FRACUNIT*P_MobjFlip(p.mo))
		end
	end
		
	if p.TRP.wallSpeed > trip.MAXCLIMB
		p.TRP.wallSpeed = $ - (p.TRP.wallTime and FRACUNIT or FRACUNIT*2)
	end

	p.TRP.sineBrake = $ or 0
	if (shouldBrake or not p.cmd.forwardmove) and not p.TRP.wallTime
		p.TRP.sineBrake = $ + FRACUNIT/18
		p.TRP.sineBrake = min($, FRACUNIT)
		p.TRP.wallSpeed = ease.inoutsine(p.TRP.sineBrake, $, 0)
	else
		p.TRP.sineBrake = $ - FRACUNIT/18
		p.TRP.sineBrake = max($, 0)
	end

	-- Left and right movement
	if p.TRP.sidemove and not shouldBrake
	and not (p.mo.flags2 & MF2_TWOD) and not (maptol & TOL_2D)
		-- Invert left and right depending on whether we go up or down
		local upDown = p.mo.momz > 0 and -1 or 1
		upDown = $ * P_MobjFlip(p.mo)
			
		-- The closer we are to our VERTICAL top speed, the faster we strafe. handled by fracMult.
		local fracMult = abs(p.TRP.wallSpeed/(trip.MAXCLIMB/FRACUNIT))
		local accel = FRACUNIT * -p.TRP.sidemove
		p.TRP.wallStrafe = trip.Clamp( $ + FixedMul(accel, fracMult), -trip.WALLSIDE*FRACUNIT, trip.WALLSIDE*FRACUNIT)

		-- Roll!
		p.TRP.wallRoll = (p.TRP.wallStrafe * -upDown / FRACUNIT) * (trip.WALLROLL/trip.WALLSIDE)
		p.TRP.wallRoll = FixedMul($, fracMult)

		-- Move!
		p.mo.momx = $ + (p.TRP.wallStrafe/FRACUNIT) * cos(p.mo.angle + ANGLE_90)
		p.mo.momy = $ + (p.TRP.wallStrafe/FRACUNIT) * sin(p.mo.angle + ANGLE_90)
	end

	-- We aren't moving in one way or another. Reset our momentum and roll.
	if (not p.TRP.sidemove) or (not p.cmd.forwardmove)
		if p.TRP.wallStrafe > 10 then p.TRP.wallStrafe = $ - abs($)/trip.WALBRAKE
		elseif p.TRP.wallStrafe < 0 then p.TRP.wallStrafe = $ + abs($)/trip.WALBRAKE end

		if p.TRP.wallStrafe < FRACUNIT/4 and p.TRP.wallStrafe > -FRACUNIT/4 then p.TRP.wallStrafe = 0 end

		if p.TRP.wallRoll > ANG2 and p.TRP.wallRoll < ANGLE_90 then p.TRP.wallRoll = $ - ANG1*3 end
		if p.TRP.wallRoll < ANGLE_MAX - ANG2 and p.TRP.wallRoll > ANGLE_270 then p.TRP.wallRoll = $ + ANG1*3 end
		if p.TRP.wallRoll < ANG2 and p.TRP.wallRoll > ANGLE_MAX - ANG2 then p.TRP.wallRoll = 0 end
	end

	if abs(p.TRP.wallSpeed) < 8 * FRACUNIT then p.mo.anim_duration = 4
	elseif abs(p.TRP.wallSpeed) > 8 * FRACUNIT and abs(p.TRP.wallSpeed) < 12 * FRACUNIT then p.mo.anim_duration = 2
	else p.mo.anim_duration = 1 end
		
	-- Finally apply everything we changed up there
	p.mo.rollangle = p.TRP.wallRoll
	local inv = 1
	local too = (twodlevel) or (p.mo.flags2 & MF2_TWOD)
	if (p.TRP.wallreverse and p.climbing < 3 and p.cmd.forwardmove)
		if too and p.cmd.forwardmove * P_MobjFlip(p.mo) < 0
			inv = 24*FRACUNIT * P_MobjFlip(p.mo)
		else
			inv = 14*FRACUNIT * P_MobjFlip(p.mo)
		end
	end
	p.mo.momz = $ + ( FixedMul(p.TRP.wallSpeed, p.mo.scale) * P_MobjFlip(p.mo) ) - inv
end